#!/usr/bin/env python3

import sys
import argparse
from clin28tools.format import CLIN28JSON

def main():
    parser = argparse.ArgumentParser(description="Validates the JSON format for the CLIN 28 shared task", formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('files', nargs='+', help='JSON files to validate')
    args = parser.parse_args()

    valid = True
    for filename in args.files:
        thisvalid = True
        try:
            data = CLIN28JSON(filename) #validation is implied
        except Exception as e:
            print("INVALID! <- " + filename + ": ", str(e),file=sys.stderr)
            valid = False
            thisvalid = False
        if thisvalid:
            print("valid <- " + filename,file=sys.stderr)
    sys.exit(0 if valid else 1)

if __name__ == '__main__':
    main()



#!/usr/bin/env python3

import sys
import argparse
import json
from clin28tools.format import CLIN28JSON


def main():
    parser = argparse.ArgumentParser(description="CLIN 28 Evaluation script", formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('--ref', type=str,help="Reference output (JSON)", action='store',required=True)
    parser.add_argument('--out', type=str,help="System output to evaluate (JSON)", action='store',required=True)
    args = parser.parse_args()

    refdata = CLIN28JSON(args.ref)
    outdata = CLIN28JSON(args.out)


    #detection
    truepos = 0
    falsepos = 0
    falseneg = 0
    #correction
    correct = 0
    incorrect = 0

    for refcorrection in refdata.corrections():
        found = False
        for outcorrection in outdata.corrections():
            if 'span' in refcorrection and 'span' in outcorrection:
                if refcorrection['span'] == outcorrection['span']:
                    outcorrection['found'] = found = True
                    span = True
            elif 'after' in refcorrection and 'after' in outcorrection:
                if refcorrection['after'] == outcorrection['after']:
                    outcorrection['found'] = found = True
                    span = False
            if found: break #no need to look further
        if found:
            truepos += 1
            if span:
                print("[DETECTION MATCH] " + ";".join(refcorrection['span']) + ": " + " ".join([ refdata[wordid]['text'] for wordid in refcorrection['span'] ]) + " -> " + outcorrection['text'],file=sys.stderr)
            else:
                print("[DETECTION MATCH] INSERTION AFTER " + refcorrection['after'] + ": " + outcorrection['text'],file=sys.stderr)

            if refcorrection['text'] == outcorrection['text']: #case sensitive!
                correct += 1
                print("\t[CORRECT]",file=sys.stderr)
            else:
                incorrect += 1
                print("\t[INCORRECT] Should be: " + refcorrection['text'],file=sys.stderr)
        else:
            falseneg += 1
            if 'span' in refcorrection:
                print("[DETECTION MISS] " + ";".join(refcorrection['span']) + ": " + " ".join([ refdata[wordid]['text'] for wordid in refcorrection['span'] ]) + " -> " + refcorrection['text'],file=sys.stderr)
            else:
                print("[DETECTION MISS] INSERTION AFTER " + refcorrection['after'] + ": " + refcorrection['text'],file=sys.stderr)

    for outcorrection in outdata.corrections():
        if 'found' not in outcorrection:
            falsepos += 1
            if 'span' in outcorrection:
                print("[DETECTION WRONG] " + ";".join(outcorrection['span']) + ": " + " ".join([ refdata[wordid]['text'] for wordid in outcorrection['span'] ]) + " -> " + outcorrection['text'],file=sys.stderr)
            else:
                print("[DETECTION WRONG] INSERTION AFTER " + outcorrection['after'] + ": " + outcorrection['text'],file=sys.stderr)


    evaluation = {
        'detection': {
            'truepos': truepos,
            'falsepos': falsepos,
            'falseneg': falseneg,
            'precision': truepos / (truepos + falsepos),
            'recall': truepos / (truepos + falseneg),
        },
        'correction': {
            'correct': correct,
            'incorrect': incorrect,
            'precision': correct / (truepos + falsepos),
            'recall': correct / (truepos + falseneg),
        },
    }
    if evaluation['detection']['precision'] + evaluation['detection']['recall'] > 0:
        evaluation['detection']['f1score'] = 2 * ((evaluation['detection']['precision'] * evaluation['detection']['recall']) / (evaluation['detection']['precision'] + evaluation['detection']['recall']))
    if evaluation['correction']['precision'] + evaluation['correction']['recall'] > 0:
        evaluation['correction']['f1score'] = 2 * ((evaluation['correction']['precision'] * evaluation['correction']['recall']) / (evaluation['correction']['precision'] + evaluation['correction']['recall']))

    print(json.dumps(evaluation, indent=4))



if __name__ == '__main__':
    main()



#!/usr/bin/env python3

import sys
import argparse
import json
from pynlpl.formats import folia


def all_original_words(doc):
    for word in doc.select(folia.Word,ignore=False): #really iterate over all words, not just the authoritative ones
        if not any( isinstance(ancestor, (folia.New, folia.Current)) for ancestor in word.ancestors() ): #exclude words that are part of a structural correction
            yield word

def folia2json(doc, docorrections=True):
    words = []
    corrections = []
    for word in all_original_words(doc):
        words.append({'id':word.id, 'text': word.text(correctionhandling=folia.CorrectionHandling.ORIGINAL), 'space': word.space, 'in':word.ancestor(folia.AbstractStructureElement).id})
    if docorrections:
        for correction in doc.select(folia.Correction):
            structural =  not any( isinstance(ancestor, folia.Word) for ancestor in correction.ancestors() ) #is the correction confined to a single word/token or is it structural?
            print("correction ", correction.id, "structural=", structural,file=sys.stderr)
            if structural:
                span = []
                try:
                    for origword in correction.original().select(folia.Word,ignore=False):
                        span.append(origword.id)
                except folia.NoSuchAnnotation:
                    pass
                if not span:
                    #we have an insertion
                    previous = correction.previous(folia.Word).id
            else:
                span = [correction.parent.id]
            try:
                text = correction.text()
            except folia.NoSuchText:
                #deletion
                text = ""
            if span:
                corrections.append({'class': correction.cls, 'span': span,'text':text })
            else:
                corrections.append({'class': correction.cls, 'after': previous,'text':text })
    return {'words':words, 'corrections': corrections}

def main():
    parser = argparse.ArgumentParser(description="Convert FoLiA to JSON Shared Task format", formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-C',dest='corrections',help="Strip corrections", action='store_false',default=True)
    parser.add_argument('file', nargs=1, help='FoLiA Document (input)')
    args = parser.parse_args()

    doc = folia.Document(file=args.file[0])
    data = folia2json(doc, args.corrections)
    print(json.dumps(data, ensure_ascii=False, indent=4))

if __name__ == '__main__':
    main()








import sys
import os
import codecs
import json

class ValidationError(Exception):
    pass

class CLIN28JSON:
    def __init__(self, filename):
        if not os.path.exists(filename):
            raise FileExistsError("File not found: " + filename)

        with open(filename,'rb') as f:
            reader = codecs.getreader('utf-8')
            try:
                self.data = json.load(reader(f))
            except:
                raise ValidationError("File is not valid JSON!")

        self.index = {}
        self.validate()

    def validate(self):
        self.index = {}
        if 'words' not in self.data:
            return ValidationError("No words found")
        if 'corrections' not in self.data:
            return ValidationError("No corrections found")
        for key in self.data:
            if key not in ('words','corrections'):
                print("WARNING: Unknown key '" + key + "' will be ignored!",file=sys.stderr)
        for word in self.words():
            if 'id' not in word or not word['id']:
                raise ValidationError("Word does not have an ID! " + repr(word))
            self.index[word['id']] = word
            if 'text' not in word or not word['text']:
                raise ValidationError("Word does not have a text! " + repr(word))
            for key in word:
                if key not in ('text','id','space','in'):
                    print("WARNING: Unknown key '" + key + "' for word " + repr(word) + " will be ignored!",file=sys.stderr)
        for correction in self.corrections():
            if 'span' not in correction or not correction['span']:
                if 'after' not in correction or not correction['after']:
                    raise ValidationError("Correction does not have a 'span' (or 'after') property! " + repr(correction))
                elif correction['after'] not in self.index:
                    raise ValidationError("Correction's 'after' property refers to a non-existing word ID! (" + correction['after'] + ") " + repr(correction))
            else:
                for wordid in correction['span']:
                    if wordid not in self.index:
                        raise ValidationError("Correction's 'span' property refers to a non-existing word ID! (" + wordid + ") " + repr(correction))
            for key in correction:
                if key not in ('text','span','after','confidence','class'):
                    print("WARNING: Unknown key '" + key + "' for correction " + repr(correction) + " will be ignored!",file=sys.stderr)
                if key == 'confidence':
                    try:
                        correction['confidence'] = float(correction['confidence'])
                    except:
                        raise ValidationError("Invalid confidence value (" + str(correction['confidence']) + ") " + repr(correction))
                    if correction['confidence'] < 0 or correction['confidence'] > 1:
                        raise ValidationError("Confidence value out of bounds (" + str(correction['confidence']) + ") " + repr(correction))

    def words(self):
        for word in self.data['words']:
            yield word

    def corrections(self):
        for correction in self.data['corrections']:
            yield correction


    def __iter__(self):
        for key in self.data:
            yield key

    def items(self):
        return self.data.items()
    def keys(self):
        return self.data.keys()
    def values(self):
        return self.data.values()

    def __len__(self):
        return len(self.data)

    def __getitem__(self, key):
        if key in self.index:
            return self.index[key]
        return self.data[key]







#! /usr/bin/env python
# -*- coding: utf8 -*-

from __future__ import print_function

import os
from setuptools import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "clin28tools",
    version = "0.1.4",
    author = "Maarten van Gompel, Merijn Beeksma, Florian Kunneman",
    author_email = "proycon@anaproy.nl",
    description = ("Scripts for the CLIN28 Shared Task on spelling correction"),
    license = "GPL",
    keywords = "nlp computational_linguistics spelling_correction",
    url = "https://github.com/LanguageMachines/CLIN28_ST_spelling_correction",
    packages=['clin28tools'],
    long_description=read('README.md'),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Topic :: Text Processing :: Linguistic",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Operating System :: POSIX",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    ],
    zip_safe=False,
    include_package_data=True,
    package_data = { },
    install_requires=[ 'pynlpl'],
    entry_points = {    'console_scripts': [
        'clin28-validator = clin28tools.validator:main' ,
        'clin28-folia2json = clin28tools.folia2json:main' ,
        'clin28-evaluate = clin28tools.evaluate:main' ,
    ] }
)
